package com.swagger.rest.repositories

import com.swagger.rest.models.ProjectMember
import com.swagger.rest.models.User
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.data.jpa.repository.JpaSpecificationExecutor
import org.springframework.data.jpa.repository.Query

interface MemberRepository : JpaRepository<ProjectMember, Long>, JpaSpecificationExecutor<ProjectMember> {

    @Query("SELECT f.* FROM project_member f WHERE project_id=?1 and role=?2 and user_id=?3", nativeQuery = true)
    fun findDup(project: String?, role: String?, user: String?): List<ProjectMember>

    @Query("SELECT f.* FROM project_member f WHERE user_id=?1", nativeQuery = true)
    fun findByUser(user: String?): List<ProjectMember>

    @Query(
        "SELECT COUNT(0) FROM project_member WHERE project_id like %?1% AND user_id=?2 AND role=?3",
        nativeQuery = true
    )
    fun validRole(project: String?, user: String?, role: String?): Int

    @Query(
        "SELECT COUNT(0) FROM project_member WHERE project_id=?1 AND user_id=?2 AND role<>2 AND role=?3",
        nativeQuery = true
    )
    fun targetRole(project: String?, user: String?, role: String?): Int

    @Query("SELECT COUNT(0) FROM project_member WHERE project_id=?1 AND user_id=?2", nativeQuery = true)
    fun validMember(project: String?, user: String?): Int

    @Query("SELECT COUNT(0) FROM project_member WHERE project_id IN (SELECT project_id FROM platform WHERE NAME=?2) AND user_id=?1 AND role=?3", nativeQuery = true)
    fun validRoleByPlatform(user: String, platform: String, role: String): Int

    @Query("SELECT member_id FROM project_member", nativeQuery = true)
    fun getId():List<Long>

    @Query("SELECT COUNT(0) FROM project_member WHERE user_id=?1 AND role=2", nativeQuery = true)
    fun validAdmin(user: String): Int
}