package com.swagger.rest.repositories

import com.swagger.rest.models.Platform
import com.swagger.rest.models.User
import org.springframework.data.domain.Page
import org.springframework.data.domain.Pageable
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.data.jpa.repository.JpaSpecificationExecutor
import org.springframework.data.jpa.repository.Query

interface PlatformRepository : JpaRepository<Platform, Long>, JpaSpecificationExecutor<Platform> {
    @Query("SELECT f.* FROM platform f WHERE name=?1 AND project_id=?2", nativeQuery = true)
    fun findByName(name: String?, project: String?): List<Platform>

    @Query("SELECT f.* FROM platform f WHERE project_id=?1", nativeQuery = true)
    fun findByProjectId(project: String?): List<Platform>

    @Query(
        "SELECT * FROM platform WHERE project_id like %?2% AND (project_id IN (SELECT project_id FROM project WHERE OWNER=?1) OR project_id IN (SELECT project_id FROM project_member WHERE user_id=?1))",
        nativeQuery = true
    )
    fun findOwnerOrMember(user: String, project: String?, pageable: Pageable): Page<Platform>

    @Query(
        "SELECT * FROM platform WHERE name=?2 AND project_id IN (SELECT project_id FROM project WHERE owner=?1 UNION SELECT project_id FROM project_member WHERE user_id=?1)",
        nativeQuery = true
    )
    fun findPlatform(user: String, platform: String): Platform

    @Query(
        "SELECT COUNT(0) FROM platform WHERE name=?2 AND project_id IN (SELECT project_id FROM project WHERE owner=?1 UNION SELECT project_id FROM project_member WHERE user_id=?1)",
        nativeQuery = true
    )
    fun findPlatform2(user: String, platform: String): Int

    @Query("SELECT platform_id FROM platform", nativeQuery = true)
    fun getId():List<Long>

}