package com.swagger.rest.repositories

import com.swagger.rest.models.Project
import com.swagger.rest.models.User
import org.springframework.data.domain.Page
import org.springframework.data.domain.Pageable
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.data.jpa.repository.JpaSpecificationExecutor
import org.springframework.data.jpa.repository.Query


interface ProjectRepository : JpaRepository<Project, Long>, JpaSpecificationExecutor<Project> {

    @Query(
        "SELECT * FROM project f WHERE OWNER=?1 OR (project_id IN (SELECT project_id FROM project_member WHERE user_id=?1))",
        nativeQuery = true
    )
    fun findOwnerOrMember(user: String, pageable: Pageable): Page<Project>

    fun findByName(name: String?): Project

    @Query("SELECT COUNT(0) FROM project WHERE name=?1", nativeQuery = true)
    fun countByName(name: String?): Int

    @Query("SELECT COUNT(0) FROM project WHERE project_id=?1 AND owner=?2", nativeQuery = true)
    fun validOwner(project: String?, owner: String?): Int

    @Query("SELECT a.project_id FROM project a", nativeQuery = true)
    fun getId():List<Long>

}